==> fintix_modelcurator-0.1.18.tar.gz: fintix_modelcurator-0.1.18/fintix_modelcurator/repository.py <==
import logging
import subprocess
import importlib
import importlib.machinery
from urllib.parse import urlparse

import psycopg2
from psycopg2 import pool

from fintix_modelcurator.const import *
from fintix_modelcurator.exit_code import *
from fintix_modelcurator.phase import *
from fintix_modelcurator.config import Config
from fintix_modelcurator.error import FintixError
from fintix_modelcurator.utils import *


class ModelRepository:
    INSTANCE = None
    CONN_POOL_MIN = 1
    CONN_POOL_MAX = 5
    MODEL_REPO_TABLE = f"{DB_SCHEMA}.custom_model"

    def __init__(self):
        self.config = Config.getInstance()
        self.db_url, _ = self.config.get(DB_URL_KEY)
        self.db_user, _ = self.config.get(DB_USER_KEY)
        self.db_pwd, _ = self.config.get(DB_PWD_KEY)

        url_parsed = urlparse(self.db_url.replace("jdbc:", ""))
        self.conpool = psycopg2.pool.ThreadedConnectionPool(
            minconn=self.CONN_POOL_MIN,
            maxconn=self.CONN_POOL_MAX,
            user=self.db_user,
            password=self.db_pwd,
            database=url_parsed.path[1:],
            host=url_parsed.hostname,
            port=url_parsed.port
        )

    def __download_model(self, model_name, phase):
        conn = None
        cur = None
        try:
            conn = self.conpool.getconn()
            cur = conn.cursor()
            if conn:
                cur.execute(f"SELECT model from io_fintix.custom_model where model_name = '{model_name}'")
                blob = cur.fetchone()
                output_dir = os.path.join(LOCAL_DEPLOY_MODEL_REPO, model_name)
                output_file = os.path.join(output_dir, model_name + ".py")
                if not os.path.exists(output_dir):
                    os.makedirs(output_dir)
                with open(output_file, "wb") as modelzip:
                    modelzip.write(blob[0])
                conn.commit()
            logging.info(f"model {model_name} have bean downloaded into {output_dir}")
        except Exception as e:
            logging.error(f"failed to download model {model_name}", e)
            return (
                NoResult,
                FintixError(exception=e, message=f"could not download model {model_name} from model repository",
                            exit_code=MODEL_DOWNLOADING_ERROR, should_exit=True)
            )
        finally:
            if cur is not None:
                try:
                    cur.close()
                except:
                    pass
            if conn is not None:
                try:
                    self.conpool.putconn(conn)
                except:
                    pass
        return NoResult, NoError

    def __convert_model(self, model_name):
        try:
            current_dir = os.getcwd()
            model_name = model_name if not model_name.endswith(".ipynb") else model_name.replace(".ipynb", "")
            model_notebook_path = os.path.join(current_dir, model_name + ".ipynb")
            res = subprocess.getoutput(f"jupyter nbconvert --to script {model_notebook_path}")
            logging.info(f"Successfully convert model '{model_name}' notebook into python executable script")
            return res, NoError
        except Exception as e:
            logging.error(f"Failed to convert notebook of model {model_name}", e)
            return (
                NoResult,
                FintixError(exception=e, message="failed to convert notebook into executable python script",
                            exit_code=None, should_exit=False)
            )

    def upload_model(self, model_name):
        res, err = self.__convert_model(model_name=model_name)
        if err is not NoError:
            handle_error(err)
        else:
            logging.info(res)

        python_script_content = None
        try:
            current_dir = os.getcwd()
            model_name = model_name if not model_name.endswith(".ipynb") else model_name.replace(".ipynb", "")
            python_script_path = os.path.join(current_dir, model_name + ".py")
            with open(python_script_path, "rb") as python_script:
                python_script_content = python_script.read()
            if python_script_content is None:
                raise Exception("failed to read converted python executable")
        except Exception as e:
            logging.error(f"Failed to read converted python script of model '{model_name}' notebook", e)
            return (
                NoResult,
                FintixError(exception=e, message="failed to zip converted python executable",
                            exit_code=None, should_exit=False)
            )

        conn = None
        cur = None
        try:
            conn = self.conpool.getconn()
            cur = conn.cursor()
            script_content = psycopg2.Binary(python_script_content)
            cur.execute(f"INSERT INTO io_fintix.custom_model (model_uid, model_name, model) VALUES({model_name}, {model_name}, %s) ON CONFLICT (model_uid) DO UPDATE SET model = %s WHERE model_uid = '{model_name}'",
                        (script_content, script_content))
            conn.commit()
            logging.info(f"Successfully upload converted python script of model '{model_name}' notebook into database")
        except Exception as e:
            logging.error(f"Failed to upload converted python script of model '{model_name}' notebook into database", e)
            return (
                NoResult,
                FintixError(exception=e, message="failed to upload converted python to database", exit_code=None,
                            should_exit=False)
            )
        finally:
            if cur is not None:
                try:
                    cur.close()
                except:
                    pass
            if conn is not None:
                try:
                    self.conpool.putconn(conn)
                except:
                    pass

        return NoResult, NoError

    def import_model(self, model_name, phase):
        if model_name is None or phase is None:
            return NoResult, FintixError(exception=None,
                                         message="failed to import model, model or phase could not be done")

        _, err = self.__download_model(model_name, phase)
        if err:
            return NoResult, err
        model_dir = os.path.join(LOCAL_DEPLOY_MODEL_REPO, model_name)
        try:
            loader = importlib.machinery.SourceFileLoader(model_name, os.path.join(model_dir, model_name + ".py"))
            model = loader.load_module(model_name)
            logging.info(f"Successfully load model '{model_name}' module from {model_dir}")
        except Exception as e:
            logging.error(f"Failed to load model '{model_name}' module form {model_dir}", e)
            return (
                NoResult,
                FintixError(
                    exception=e, message=f"failed to import model in {model_dir}",
                    exit_code=MODEL_CODE_IMPORT_ERR, should_exit=True)
            )

        return model, NoError

    @classmethod
    def getInstance(cls):
        if cls.INSTANCE is None:
            cls.INSTANCE = ModelRepository()
        return cls.INSTANCE

==> fintix_modelcurator-0.1.19.tar.gz: fintix_modelcurator-0.1.19/fintix_modelcurator/repository.py <==
import logging
import subprocess
import importlib
import importlib.machinery
from urllib.parse import urlparse

import psycopg2
from psycopg2 import pool

from fintix_modelcurator.const import *
from fintix_modelcurator.exit_code import *
from fintix_modelcurator.phase import *
from fintix_modelcurator.config import Config
from fintix_modelcurator.error import FintixError
from fintix_modelcurator.utils import *


class ModelRepository:
    INSTANCE = None
    CONN_POOL_MIN = 1
    CONN_POOL_MAX = 5
    MODEL_REPO_TABLE = f"{DB_SCHEMA}.custom_model"

    def __init__(self):
        self.config = Config.getInstance()
        self.db_url, _ = self.config.get(DB_URL_KEY)
        self.db_user, _ = self.config.get(DB_USER_KEY)
        self.db_pwd, _ = self.config.get(DB_PWD_KEY)

        url_parsed = urlparse(self.db_url.replace("jdbc:", ""))
        self.conpool = psycopg2.pool.ThreadedConnectionPool(
            minconn=self.CONN_POOL_MIN,
            maxconn=self.CONN_POOL_MAX,
            user=self.db_user,
            password=self.db_pwd,
            database=url_parsed.path[1:],
            host=url_parsed.hostname,
            port=url_parsed.port
        )

    def __download_model(self, model_name, phase):
        conn = None
        cur = None
        try:
            conn = self.conpool.getconn()
            cur = conn.cursor()
            if conn:
                cur.execute(f"SELECT model from io_fintix.custom_model where model_name = '{model_name}'")
                blob = cur.fetchone()
                output_dir = os.path.join(LOCAL_DEPLOY_MODEL_REPO, model_name)
                output_file = os.path.join(output_dir, model_name + ".py")
                if not os.path.exists(output_dir):
                    os.makedirs(output_dir)
                with open(output_file, "wb") as modelzip:
                    modelzip.write(blob[0])
                conn.commit()
            logging.info(f"model {model_name} have bean downloaded into {output_dir}")
        except Exception as e:
            logging.error(f"failed to download model {model_name}", e)
            return (
                NoResult,
                FintixError(exception=e, message=f"could not download model {model_name} from model repository",
                            exit_code=MODEL_DOWNLOADING_ERROR, should_exit=True)
            )
        finally:
            if cur is not None:
                try:
                    cur.close()
                except:
                    pass
            if conn is not None:
                try:
                    self.conpool.putconn(conn)
                except:
                    pass
        return NoResult, NoError

    def __convert_model(self, model_name):
        try:
            current_dir = os.getcwd()
            model_name = model_name if not model_name.endswith(".ipynb") else model_name.replace(".ipynb", "")
            model_notebook_path = os.path.join(current_dir, model_name + ".ipynb")
            res = subprocess.getoutput(f"jupyter nbconvert --to script {model_notebook_path}")
            logging.info(f"Successfully convert model '{model_name}' notebook into python executable script")
            return res, NoError
        except Exception as e:
            logging.error(f"Failed to convert notebook of model {model_name}", e)
            return (
                NoResult,
                FintixError(exception=e, message="failed to convert notebook into executable python script",
                            exit_code=None, should_exit=False)
            )

    def upload_model(self, model_name):
        res, err = self.__convert_model(model_name=model_name)
        if err is not NoError:
            handle_error(err)
        else:
            logging.info(res)

        python_script_content = None
        try:
            current_dir = os.getcwd()
            model_name = model_name if not model_name.endswith(".ipynb") else model_name.replace(".ipynb", "")
            python_script_path = os.path.join(current_dir, model_name + ".py")
            with open(python_script_path, "rb") as python_script:
                python_script_content = python_script.read()
            if python_script_content is None:
                raise Exception("failed to read converted python executable")
        except Exception as e:
            logging.error(f"Failed to read converted python script of model '{model_name}' notebook", e)
            return (
                NoResult,
                FintixError(exception=e, message="failed to zip converted python executable",
                            exit_code=None, should_exit=False)
            )

        conn = None
        cur = None
        try:
            conn = self.conpool.getconn()
            cur = conn.cursor()
            script_content = psycopg2.Binary(python_script_content)
            cur.execute(f"INSERT INTO io_fintix.custom_model (model_uid, model_name, model) VALUES('{model_name}', '{model_name}', %s) ON CONFLICT (model_uid) DO UPDATE SET model = %s WHERE model_uid = '{model_name}'",
                        (script_content, script_content))
            conn.commit()
            logging.info(f"Successfully upload converted python script of model '{model_name}' notebook into database")
        except Exception as e:
            logging.error(f"Failed to upload converted python script of model '{model_name}' notebook into database", e)
            return (
                NoResult,
                FintixError(exception=e, message="failed to upload converted python to database", exit_code=None,
                            should_exit=False)
            )
        finally:
            if cur is not None:
                try:
                    cur.close()
                except:
                    pass
            if conn is not None:
                try:
                    self.conpool.putconn(conn)
                except:
                    pass

        return NoResult, NoError

    def import_model(self, model_name, phase):
        if model_name is None or phase is None:
            return NoResult, FintixError(exception=None,
                                         message="failed to import model, model or phase could not be done")

        _, err = self.__download_model(model_name, phase)
        if err:
            return NoResult, err
        model_dir = os.path.join(LOCAL_DEPLOY_MODEL_REPO, model_name)
        try:
            loader = importlib.machinery.SourceFileLoader(model_name, os.path.join(model_dir, model_name + ".py"))
            model = loader.load_module(model_name)
            logging.info(f"Successfully load model '{model_name}' module from {model_dir}")
        except Exception as e:
            logging.error(f"Failed to load model '{model_name}' module form {model_dir}", e)
            return (
                NoResult,
                FintixError(
                    exception=e, message=f"failed to import model in {model_dir}",
                    exit_code=MODEL_CODE_IMPORT_ERR, should_exit=True)
            )

        return model, NoError

    @classmethod
    def getInstance(cls):
        if cls.INSTANCE is None:
            cls.INSTANCE = ModelRepository()
        return cls.INSTANCE
